--Corner Pin effect filter by khaver

obs = obslua
bit = require("bit")

SETTING_TLX = 'pos_tlx'
SETTING_TLY = 'pos_tly'
SETTING_TRX = 'pos_trx'
SETTING_TRY = 'pos_try'
SETTING_BLX = 'pos_blx'
SETTING_BLY = 'pos_bly'
SETTING_BRX = 'pos_brx'
SETTING_BRY = 'pos_bry'

TEXT_TLX = 'Top Left X'
TEXT_TLY = 'Top Left Y'
TEXT_TRX = 'Top Right X'
TEXT_TRY = 'Top Right Y'
TEXT_BLX = 'Bottom Left X'
TEXT_BLY = 'Bottom Left Y'
TEXT_BRX = 'Bottom Right X'
TEXT_BRY = 'Bottom Right Y'

source_def = {}
source_def.id = 'filter-cornerpin'
source_def.type = obs.OBS_SOURCE_TYPE_FILTER
source_def.output_flags = bit.bor(obs.OBS_SOURCE_VIDEO)

function set_render_size(filter)
    target = obs.obs_filter_get_target(filter.context)

    local width, height
    if target == nil then
        width = 0
        height = 0
    else
        width = obs.obs_source_get_base_width(target)
        height = obs.obs_source_get_base_height(target)
    end

    filter.width = width
    filter.height = height
end

source_def.get_name = function()
    return "Corner Pin"
end

source_def.create = function(settings, source)
    local effect_path = script_path() .. 'filter-cornerpin/filter-cornerpin.effect'

    filter = {}
    filter.params = {}
    filter.context = source

    set_render_size(filter)

    obs.obs_enter_graphics()
    filter.effect = obs.gs_effect_create_from_file(effect_path, nil)
    if filter.effect ~= nil then
        filter.params.pos_tlx = obs.gs_effect_get_param_by_name(filter.effect, 'pos_tlx')
        filter.params.pos_tly = obs.gs_effect_get_param_by_name(filter.effect, 'pos_tly')
        filter.params.pos_trx = obs.gs_effect_get_param_by_name(filter.effect, 'pos_trx')
        filter.params.pos_try = obs.gs_effect_get_param_by_name(filter.effect, 'pos_try')
        filter.params.pos_blx = obs.gs_effect_get_param_by_name(filter.effect, 'pos_blx')
        filter.params.pos_bly = obs.gs_effect_get_param_by_name(filter.effect, 'pos_bly')
        filter.params.pos_brx = obs.gs_effect_get_param_by_name(filter.effect, 'pos_brx')
        filter.params.pos_bry = obs.gs_effect_get_param_by_name(filter.effect, 'pos_bry')
    end
    obs.obs_leave_graphics()

    if filter.effect == nil then
        source_def.destroy(filter)
        return nil
    end

    source_def.update(filter, settings)
    return filter
end

source_def.destroy = function(filter)
    if filter.effect ~= nil then
        obs.obs_enter_graphics()
        obs.gs_effect_destroy(filter.effect)
        obs.obs_leave_graphics()
    end
end

source_def.get_width = function(filter)
    return filter.width
end

source_def.get_height = function(filter)
    return filter.height
end

source_def.update = function(filter, settings)
    filter.pos_tlx = obs.obs_data_get_double(settings, SETTING_TLX)
    filter.pos_tly = obs.obs_data_get_double(settings, SETTING_TLY)
    filter.pos_trx = obs.obs_data_get_double(settings, SETTING_TRX)
    filter.pos_try = obs.obs_data_get_double(settings, SETTING_TRY)
    filter.pos_blx = obs.obs_data_get_double(settings, SETTING_BLX)
    filter.pos_bly = obs.obs_data_get_double(settings, SETTING_BLY)
    filter.pos_brx = obs.obs_data_get_double(settings, SETTING_BRX)
    filter.pos_bry = obs.obs_data_get_double(settings, SETTING_BRY)

    set_render_size(filter)
end

source_def.video_render = function(filter, effect)
    obs.obs_source_process_filter_begin(filter.context, obs.GS_RGBA, obs.OBS_NO_DIRECT_RENDERING)

    obs.gs_effect_set_float(filter.params.pos_tlx, filter.pos_tlx)
    obs.gs_effect_set_float(filter.params.pos_tly, filter.pos_tly)
    obs.gs_effect_set_float(filter.params.pos_trx, filter.pos_trx)
    obs.gs_effect_set_float(filter.params.pos_try, filter.pos_try)
    obs.gs_effect_set_float(filter.params.pos_blx, filter.pos_blx)
    obs.gs_effect_set_float(filter.params.pos_bly, filter.pos_bly)
    obs.gs_effect_set_float(filter.params.pos_brx, filter.pos_brx)
    obs.gs_effect_set_float(filter.params.pos_bry, filter.pos_bry)

    obs.obs_source_process_filter_end(filter.context, filter.effect, filter.width, filter.height)
end

source_def.get_properties = function(settings)
    props = obs.obs_properties_create()

    obs.obs_properties_add_float_slider(props, SETTING_TLX, TEXT_TLX, 0.0, 1.0, 0.0001)
    obs.obs_properties_add_float_slider(props, SETTING_TLY, TEXT_TLY, 0.0, 1.0, 0.0001)
    obs.obs_properties_add_float_slider(props, SETTING_TRX, TEXT_TRX, 0.0, 1.0, 0.0001)
    obs.obs_properties_add_float_slider(props, SETTING_TRY, TEXT_TRY, 0.0, 1.0, 0.0001)
    obs.obs_properties_add_float_slider(props, SETTING_BLX, TEXT_BLX, 0.0, 1.0, 0.0001)
    obs.obs_properties_add_float_slider(props, SETTING_BLY, TEXT_BLY, 0.0, 1.0, 0.0001)
    obs.obs_properties_add_float_slider(props, SETTING_BRX, TEXT_BRX, 0.0, 1.0, 0.0001)
    obs.obs_properties_add_float_slider(props, SETTING_BRY, TEXT_BRY, 0.0, 1.0, 0.0001)

    return props
end

source_def.get_defaults = function(settings)
    obs.obs_data_set_default_double(settings, SETTING_TLX, 0.0)
    obs.obs_data_set_default_double(settings, SETTING_TLY, 0.0)
    obs.obs_data_set_default_double(settings, SETTING_TRX, 0.0)
    obs.obs_data_set_default_double(settings, SETTING_TRY, 0.0)
    obs.obs_data_set_default_double(settings, SETTING_BLX, 0.0)
    obs.obs_data_set_default_double(settings, SETTING_BLY, 0.0)
    obs.obs_data_set_default_double(settings, SETTING_BRX, 0.0)
    obs.obs_data_set_default_double(settings, SETTING_BRY, 0.0)
end

source_def.video_tick = function(filter, seconds)
    set_render_size(filter)
end

obs.obs_register_source(source_def)
